package com.supwisdom.institute.backend.base.domain.repo;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Predicate;
import javax.persistence.criteria.Root;

import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.util.StringUtils;

import com.supwisdom.institute.backend.base.domain.entity.Role;
import com.supwisdom.institute.backend.common.framework.repo.BaseJpaRepository;
import com.supwisdom.institute.backend.common.util.MapBeanUtils;

@Repository
public interface RoleRepository extends BaseJpaRepository<Role> {

  @Override
  public default Specification<Role> convertToSpec(Map<String, Object> mapBean) {
    
    Specification<Role> spec = new Specification<Role>() {

      /**
       * 
       */
      private static final long serialVersionUID = -263282246904382286L;

      @Override
      public Predicate toPredicate(Root<Role> root, CriteriaQuery<?> query, CriteriaBuilder criteriaBuilder) {
        List<Predicate> predicates = new ArrayList<>();
        
        if (mapBean != null) {

          if (MapBeanUtils.getBoolean(mapBean, "deleted") != null) {
            predicates.add(criteriaBuilder.equal(root.get("deleted"), MapBeanUtils.getBoolean(mapBean, "deleted")));
          }

          if (!StringUtils.isEmpty(MapBeanUtils.getString(mapBean, "code"))) {
            predicates.add(criteriaBuilder.like(root.get("code"), "%" + MapBeanUtils.getString(mapBean, "code") + "%"));
          }
          if (!StringUtils.isEmpty(MapBeanUtils.getString(mapBean, "name"))) {
            predicates.add(criteriaBuilder.like(root.get("name"), "%" + MapBeanUtils.getString(mapBean, "name") + "%"));
          }
          if (!StringUtils.isEmpty(MapBeanUtils.getString(mapBean, "memo"))) {
            predicates.add(criteriaBuilder.like(root.get("memo"), "%" + MapBeanUtils.getString(mapBean, "memo") + "%"));
          }
          
          if (!StringUtils.isEmpty(MapBeanUtils.getString(mapBean, "status"))) {
            predicates.add(criteriaBuilder.equal(root.get("status"), MapBeanUtils.getString(mapBean, "status")));
          }

          List<Predicate> predicatesKeyword = new ArrayList<>();
          if (!StringUtils.isEmpty(MapBeanUtils.getString(mapBean, "keyword"))) {
            predicatesKeyword.add(criteriaBuilder.like(root.get("code"), "%" + MapBeanUtils.getString(mapBean, "keyword") + "%"));
            predicatesKeyword.add(criteriaBuilder.like(root.get("name"), "%" + MapBeanUtils.getString(mapBean, "keyword") + "%"));
            predicatesKeyword.add(criteriaBuilder.like(root.get("memo"), "%" + MapBeanUtils.getString(mapBean, "keyword") + "%"));
            
            predicates.add(criteriaBuilder.or(predicatesKeyword.toArray(new Predicate[predicatesKeyword.size()])));
          }
        }
        
        return criteriaBuilder.and(predicates.toArray(new Predicate[predicates.size()]));
      }
      
    };
    
    return spec;
  }

//  @Override
//  public default Page<Role> selectPageList(boolean loadAll, int pageIndex, int pageSize, Map<String, Object> mapBean, Map<String, String> orderBy) {
//    if (loadAll) {
//      pageIndex = 0;
//      pageSize = Integer.MAX_VALUE;
//    }
//    
//    Role probe = new Role();
//    if (mapBean != null) {
//      probe.setDeleted(MapBeanUtils.getBoolean(mapBean, "deleted"));
//      probe.setCode(MapBeanUtils.getString(mapBean, "code"));
//      probe.setName(MapBeanUtils.getString(mapBean, "name"));
//      probe.setMemo(MapBeanUtils.getString(mapBean, "memo"));
//      probe.setStatus(MapBeanUtils.getString(mapBean, "status"));
//    }
//    
//    ExampleMatcher matcher = ExampleMatcher.matching()
//        .withMatcher("deleted", ExampleMatcher.GenericPropertyMatchers.exact())
//        .withMatcher("code", ExampleMatcher.GenericPropertyMatchers.contains())
//        .withMatcher("name", ExampleMatcher.GenericPropertyMatchers.contains())
//        .withMatcher("memo", ExampleMatcher.GenericPropertyMatchers.contains())
//        .withMatcher("status", ExampleMatcher.GenericPropertyMatchers.exact());
//    
//    PageRequest pageRequest = PageRequest.of(pageIndex, pageSize);
//    Example<Role> example = Example.of(probe, matcher);
//    
//    Page<Role> page = this.findAll(example, pageRequest);
//    
//    return page;
//  }
  
  
  public default Role selectByCode(String code) {
    Role probe = new Role();
    probe.setCode(code);
    
    ExampleMatcher matcher = ExampleMatcher.matching()
        .withMatcher("code", ExampleMatcher.GenericPropertyMatchers.exact());
    
    Example<Role> example = Example.of(probe, matcher);
    
    Optional<Role> o = this.findOne(example);
    
    if (o.isPresent()) {
      return o.get();
    }
    
    return null;
  }
  
  @Query(value = "select r from Role r "
      + "inner join AccountRole ar on r.id=ar.roleId "
      + "inner join Account a on ar.accountId=a.id "
      + "where a.username=:username "
      + "and r.status='1' and a.status='1' and a.enabled=1 ")
  public List<Role> selectAccountRoleByUsername(@Param("username") String username);
  
  @Query(value = "select r from Role r "
      + "inner join GroupRole gr on r.id=gr.roleId "
      + "inner join Group_ g on gr.groupId=g.id "
      + "inner join AccountGroup ag on g.id=ag.groupId "
      + "inner join Account a on ag.accountId=a.id "
      + "where a.username=:username "
      + "and r.status='1' and g.status='1' and a.status='1' and a.enabled=1 ")
  public List<Role> selectAccountGroupRoleByUsername(@Param("username") String username);

  
  @Query(value = "select r from Role r "
      + "inner join RolePermission rp on r.id=rp.roleId "
      + "inner join Permission p on rp.permissionId=p.id "
      + "where p.id=:permissionId "
      + "and r.status='1' and p.status='1' ")
  public List<Role> selectPermissionRolesByPermission(@Param("permissionId") String permissionId);
  
  
  @Query(value = "select r from Role r "
      + "inner join RolePermission rp on r.id=rp.roleId "
      + "inner join Permission p on rp.permissionId=p.id "
      + "inner join PermissionResource pres on p.id=pres.permissionId "
      + "inner join Resource res on pres.resourceId=res.id "
      + "where res.id=:resourceId "
      + "and r.status='1' and p.status='1' and res.status='1' ")
  public List<Role> selectResourceRolesByResource(@Param("resourceId") String resourceId);
  
}
